////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire.authenticator;

import org.watto.program.android.sync.xfire.GlobalConstants;
import org.watto.program.android.sync.xfire.R;
import android.accounts.AbstractAccountAuthenticator;
import android.accounts.Account;
import android.accounts.AccountAuthenticatorResponse;
import android.accounts.AccountManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;


/***********************************************************************************************
An <code>AbstractAccountAuthenticator</code> for authenticating accounts in the 
<i>org.watto.program.android.sync.xfire</i> domain.
***********************************************************************************************/
class AccountAuthenticator extends AbstractAccountAuthenticator {

  /** The Authentication Service context **/
  Context context;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public AccountAuthenticator(Context context){
    super(context);
    // TODO REMOVE THIS LINE!!!
    //android.os.Debug.waitForDebugger();
    this.context = context;
  }


  /***********************************************************************************************
  Adds an account of the specified accountType.
  @param response to send the result back to the AccountManager, will never be null 
  @param accountType the type of account to add, will never be null 
  @param authTokenType the type of auth token to retrieve after adding the account, may be null 
  @param requiredFeatures a String array of authenticator-specific features that the added account
  must support, may be null 
  @param options a Bundle of authenticator-specific options, may be null 
  @return a Bundle result or null if the result is to be returned via the response. The result will
  contain either: KEY_INTENT, or KEY_ACCOUNT_NAME and KEY_ACCOUNT_TYPE of the account that was added,
  or KEY_ERROR_CODE and KEY_ERROR_MESSAGE to indicate an error 
  ***********************************************************************************************/
  public Bundle addAccount(AccountAuthenticatorResponse response,String accountType,String authTokenType,String[] requiredFeatures,Bundle options){
    // Creates an Intent that says that we will call the AuthenticatorCallback to create the account
    Intent intent = new Intent(context,AuthenticatorActivity.class);
    intent.putExtra(AccountManager.KEY_ACCOUNT_AUTHENTICATOR_RESPONSE,response);
    intent.putExtra("action","addAccount");
 
    Bundle bundle = new Bundle();
    bundle.putParcelable(AccountManager.KEY_INTENT,intent);
    return bundle;
  }


  /***********************************************************************************************
  Checks that the user knows the credentials of an account
  @param response to send the result back to the AccountManager, will never be null 
  @param account the account whose credentials are to be checked, will never be null 
  @param options a Bundle of authenticator-specific options, may be null 
  @return a Bundle result or null if the result is to be returned via the response. The result will
  contain either: KEY_INTENT, or KEY_BOOLEAN_RESULT, true if the check succeeded, false otherwise
  KEY_ERROR_CODE and KEY_ERROR_MESSAGE to indicate an error 
  ***********************************************************************************************/
  public Bundle confirmCredentials(AccountAuthenticatorResponse response,Account account,Bundle options){

    // If the password is supplied in the options, try to authenticate it on the server
    if (options != null && options.containsKey(AccountManager.KEY_PASSWORD)) {
      // validate the password
      boolean confirmed = XFireAuthenticator.confirmCredentials();

      // Return the result of the password validation
      Bundle resultBundle = new Bundle();
      resultBundle.putBoolean(AccountManager.KEY_BOOLEAN_RESULT,confirmed);
      return resultBundle;
    }

    // The password was not specified, so we want to call the AuthenticatorActivity
    // to ask the user to enter their password
    Intent intent = new Intent(context,AuthenticatorActivity.class);
    intent.putExtra(AccountManager.KEY_ACCOUNT_AUTHENTICATOR_RESPONSE,response);
    intent.putExtra("action","confirmCredentials");
    intent.putExtra("username",account.name);

    Bundle bundle = new Bundle();
    bundle.putParcelable(AccountManager.KEY_INTENT,intent);
    return bundle;
  }


  /***********************************************************************************************
  Returns a Bundle that contains the Intent of the activity that can be used to edit the properties.
  In order to indicate success the activity should call response.setResult() with a non-null Bundle.
  @param response used to set the result for the request. If the Constants.INTENT_KEY is set in the
  bundle then this response field is to be used for sending future results if and when the Intent
  is started. 
  @param accountType the AccountType whose properties are to be edited. 
  @return a Bundle containing the result or the Intent to start to continue the request. If this is
  null then the request is considered to still be active and the result should sent later using
  response.
  ***********************************************************************************************/
  public Bundle editProperties(AccountAuthenticatorResponse response,String accountType){
    throw new UnsupportedOperationException();
  }


  /***********************************************************************************************
  Gets the authtoken for an account.
  @param response  to send the result back to the AccountManager, will never be null 
  @param account  the account whose credentials are to be retrieved, will never be null 
  @param authTokenType  the type of auth token to retrieve, will never be null 
  @param options  a Bundle of authenticator-specific options, may be null 
  @return a Bundle result or null if the result is to be returned via the response. The result will
  contain either: KEY_INTENT, or KEY_ACCOUNT_NAME, KEY_ACCOUNT_TYPE, and KEY_AUTHTOKEN, or 
  KEY_ERROR_CODE and KEY_ERROR_MESSAGE to indicate an error 
  ***********************************************************************************************/
  public Bundle getAuthToken(AccountAuthenticatorResponse response,Account account,String authTokenType,Bundle loginOptions){
    // Check that the authTokenType being asked for is org.watto.program.android.sync.xfire
    if (!authTokenType.equals(GlobalConstants.AUTHTOKEN_TYPE)) {
      Bundle resultBundle = new Bundle();
      resultBundle.putString(AccountManager.KEY_ERROR_MESSAGE,"Invalid authTokenType");
      return resultBundle;
    }

    AccountManager accountManager = AccountManager.get(context);
    String password = accountManager.getPassword(account);

    if (password != null) {
      // If the password is specified, check that the username and password are valid
      boolean confirmed = XFireAuthenticator.confirmCredentials();
      if (confirmed) {
        // The username and password are correct, so return the result
        return XFireAuthenticator.getAccountBundle(accountManager);

      }
    }

    // The password was missing or incorrect, so call the AuthenticatorActivity to
    // ask the user to specify the password
    Intent intent = new Intent(context,AuthenticatorActivity.class);
    intent.putExtra(AccountManager.KEY_ACCOUNT_AUTHENTICATOR_RESPONSE,response);
    intent.putExtra("action","getAuthToken");
    intent.putExtra("username",account.name);

    Bundle bundle = new Bundle();
    bundle.putParcelable(AccountManager.KEY_INTENT,intent);
    return bundle;
  }


  /***********************************************************************************************
  Ask the authenticator for a localized label for the given authTokenType.
  @param authTokenType  the authTokenType whose label is to be returned, will never be null 
  @return the localized label of the auth token type, may be null if the type isn't known 
  ***********************************************************************************************/
  public String getAuthTokenLabel(String authTokenType){
    // Check that the authTokenType being asked for is org.watto.program.android.sync.xfire
    if (authTokenType.equals(GlobalConstants.AUTHTOKEN_TYPE)) {
      // Gets the user-friendly name for this account type
      return context.getString(R.string.sync_name);
    }
    return null;
  }


  /***********************************************************************************************
  Checks if the account supports all the specified authenticator specific features.
  @param response  to send the result back to the AccountManager, will never be null 
  @param account  the account to check, will never be null 
  @param features  an array of features to check, will never be null 
  @return a Bundle result or null if the result is to be returned via the response. The result will
  contain either: KEY_INTENT, or KEY_BOOLEAN_RESULT, true if the account has all the features, false
  otherwise KEY_ERROR_CODE and KEY_ERROR_MESSAGE to indicate an error 
  ***********************************************************************************************/
  public Bundle hasFeatures(AccountAuthenticatorResponse response,Account account,String[] features){
    Bundle resultBundle = new Bundle();
    resultBundle.putBoolean(AccountManager.KEY_BOOLEAN_RESULT,false);
    return resultBundle;
  }


  /***********************************************************************************************
  Update the locally stored credentials for an account.
  @param response  to send the result back to the AccountManager, will never be null 
  @param account  the account whose credentials are to be updated, will never be null 
  @param authTokenType  the type of auth token to retrieve after updating the credentials, may be
  null 
  @param options  a Bundle of authenticator-specific options, may be null 
  @return a Bundle result or null if the result is to be returned via the response. The result will
  contain either: KEY_INTENT, or KEY_ACCOUNT_NAME and KEY_ACCOUNT_TYPE of the account that was added,
  or KEY_ERROR_CODE and KEY_ERROR_MESSAGE to indicate an error 
  ***********************************************************************************************/
  public Bundle updateCredentials(AccountAuthenticatorResponse response,Account account,String authTokenType,Bundle loginOptions){
    // The username or password needs to be re-specified. So call the AuthenticatorActivity to do it.
    Intent intent = new Intent(context,AuthenticatorActivity.class);
    intent.putExtra("action","updateCredentials");
    intent.putExtra("username",account.name);

    Bundle bundle = new Bundle();
    bundle.putParcelable(AccountManager.KEY_INTENT,intent);
    return bundle;
  }

}
