////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire.authenticator;

import java.net.HttpURLConnection;
import java.net.URL;
import org.watto.program.android.sync.xfire.GlobalConstants;
import org.watto.program.android.sync.xfire.security.XFireAuthProtocol;
import org.watto.program.android.sync.xfire.sync.SyncIntervalWrapper;
import android.accounts.Account;
import android.accounts.AccountManager;
import android.app.AlarmManager;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.provider.ContactsContract;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class XFireAuthenticator {

  /** The authentication for this application **/
  public static final XFireAuthProtocol authClient = new XFireAuthProtocol();


  /***********************************************************************************************
  Authenticates the <i>username</i> and <i>password</i> on the backend server
  ***********************************************************************************************/
  public static boolean confirmCredentials(){
    try {

      if (!credentialsKnown()) {
        return false;
      }

      // The URL is the details of "myself"
      URL authURL = new URL(getFriendsUrl());
      HttpURLConnection request = (HttpURLConnection)authURL.openConnection();

      // Send the request
      request.connect();

      // Get the response code
      if (request.getResponseCode() == 200) {
        return true;
      }
      return false;

    }
    catch (Throwable t) {
      return false;
    }
  }


  /***********************************************************************************************
  Creates a new <code>Account</code> and configures automatic syncing
  @param accountManager the <code>AccountManager</code> used to create the <code>Account</code>
  ***********************************************************************************************/
  public static Account createAccount(AccountManager accountManager){
    Account account = new Account(GlobalConstants.ACCOUNT_NAME,GlobalConstants.ACCOUNT_TYPE);

    // Create the account with the code as the password
    accountManager.addAccountExplicitly(account,authClient.getUsername(),null);
    // Add the authentication token
    accountManager.setAuthToken(account,GlobalConstants.AUTHTOKEN_TYPE,authClient.getUsername());

    // Enable the auto-syncing of the account
    ContentResolver.setSyncAutomatically(account,ContactsContract.AUTHORITY,true);
    // Trigger the first sync immediately
    ContentResolver.requestSync(account,ContactsContract.AUTHORITY,new Bundle());

    return account;
  }


  /***********************************************************************************************
  Creates a new <code>Account</code> and configures automatic syncing (Android <= 2.1)
  @param accountManager the <code>AccountManager</code> used to create the <code>Account</code>
  @param alarmManager the <code>AlarmManager</code> used to schedule repeating events
  ***********************************************************************************************/
  public static void createAccount(AccountManager accountManager,AlarmManager alarmManager,Context context){
    createAccount(accountManager);

    // Repeat the sync every X minutes
    SyncIntervalWrapper.addPeriodicSync(context,GlobalConstants.DEFAULT_SYNC_INTERVAL);
  }


  /***********************************************************************************************
  Creates a new <code>Account</code> and configures automatic syncing (Android 2.2+)
  @param accountManager the <code>AccountManager</code> used to create the <code>Account</code>
  @param resolver the <code>ContentResolver</code> used to change the <code>Account</code> settings
  ***********************************************************************************************/
  public static void createAccount(AccountManager accountManager,ContentResolver resolver){
    Account account = createAccount(accountManager);

    // Repeat the sync every X minutes
    SyncIntervalWrapper.addPeriodicSync(resolver,account,ContactsContract.AUTHORITY,null,GlobalConstants.DEFAULT_SYNC_INTERVAL);
  }


  /***********************************************************************************************
  Gets whether the token and secret are specified or not
  ***********************************************************************************************/
  public static boolean credentialsKnown(){
    String customUrl = authClient.getUsername();
    if (customUrl == null || customUrl.equals("")) {

      // Try to load the access token from the Settings
      authClient.loadFromSettings();

      // Re-check the access token
      customUrl = authClient.getUsername();
      if (customUrl == null || customUrl.equals("")) {
        return false;
      }
    }
    return true;
  }


  /***********************************************************************************************
  Gets a <code>Bundle</code> for the account, containing the name, type, access token and code
  ***********************************************************************************************/
  public static Bundle getAccountBundle(AccountManager accountManager){
    Bundle bundle = new Bundle();
    bundle.putString(AccountManager.KEY_ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
    bundle.putString(AccountManager.KEY_ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    bundle.putString(AccountManager.KEY_AUTHTOKEN,authClient.getUsername());
    bundle.putString(AccountManager.KEY_PASSWORD,authClient.getUsername());
    return bundle;
  }


  /***********************************************************************************************
  Gets an <code>Intent</code> for the account, containing the name, type, access token and code
  ***********************************************************************************************/
  public static Intent getAccountIntent(AccountManager accountManager){
    Intent intent = new Intent();
    intent.putExtra(AccountManager.KEY_ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
    intent.putExtra(AccountManager.KEY_ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    intent.putExtra(AccountManager.KEY_AUTHTOKEN,authClient.getUsername());
    intent.putExtra(AccountManager.KEY_PASSWORD,authClient.getUsername());
    return intent;
  }

  
  public static void setUsername(String username){
    authClient.setUsername(username);
  }
  

  public static String getFriendsUrl(){
    return authClient.getFriendsUrl();
  }


  public static String getProfileUrl(String username){
    return authClient.getProfileUrl(username);
  }


  /***********************************************************************************************
  Updates an existing <code>Account</code> with new credentials
  ***********************************************************************************************/
  public static void updateAccount(AccountManager accountManager){
    Account account = new Account(GlobalConstants.ACCOUNT_NAME,GlobalConstants.ACCOUNT_TYPE);
    accountManager.setAuthToken(account,GlobalConstants.AUTHTOKEN_TYPE,authClient.getUsername());
    accountManager.setPassword(account,authClient.getUsername());
  }

}
