////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire.sync;

import java.util.Calendar;
import org.watto.android.R;
import org.watto.program.android.sync.xfire.BatchOperation;
import org.watto.program.android.sync.xfire.ContactOperations;
import org.watto.program.android.sync.xfire.GlobalConstants;
import org.watto.program.android.sync.xfire.XFireFriend;
import org.watto.program.android.sync.xfire.XFireQuery;
import org.watto.program.android.sync.xfire.datatype.AgeData;
import org.watto.program.android.sync.xfire.datatype.BiographyData;
import org.watto.program.android.sync.xfire.datatype.GamingStyleData;
import org.watto.program.android.sync.xfire.datatype.GenderData;
import org.watto.program.android.sync.xfire.datatype.InterestsData;
import org.watto.program.android.sync.xfire.datatype.JoinDateData;
import org.watto.program.android.sync.xfire.datatype.LocationData;
import org.watto.program.android.sync.xfire.datatype.OccupationData;
import org.watto.program.android.sync.xfire.datatype.StatusData;
import org.watto.program.android.sync.xfire.datatype.UsernameData;
import org.watto.program.android.sync.xfire.datatype.WebsiteData;
import android.accounts.Account;
import android.accounts.AccountManager;
import android.content.AbstractThreadedSyncAdapter;
import android.content.ContentProviderClient;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.Context;
import android.content.SyncResult;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.GroupMembership;
import android.provider.ContactsContract.CommonDataKinds.Nickname;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.CommonDataKinds.StructuredName;
import android.provider.ContactsContract.CommonDataKinds.StructuredPostal;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.Groups;
import android.provider.ContactsContract.RawContacts;


/***********************************************************************************************
A <code>SyncAdapter</code> implementation for syncing <code>SyncAdapter</code> contacts to the
platform <code>ContactOperations</code> provider.
***********************************************************************************************/
public class SyncAdapter extends AbstractThreadedSyncAdapter {

  /** Access to the sync accounts on this device **/
  AccountManager accountManager;
  Context context;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public SyncAdapter(Context context, boolean autoInitialize){
    super(context,autoInitialize);
    this.context = context;
    this.accountManager = AccountManager.get(context);
  }


  /***********************************************************************************************
  Performs a synchronisation for this account. SyncAdapter-specific parameters may be specified in
  extras, which is guaranteed to not be null.
  @param account the account that should be synced 
  @param extras SyncAdapter-specific parameters 
  @param authority the authority of this sync request 
  @param provider a ContentProviderClient that points to the ContentProvider for this authority 
  @param syncResult SyncAdapter-specific parameters  
  ***********************************************************************************************/
  public void onPerformSync(Account account,Bundle extras,String authority,ContentProviderClient provider,SyncResult syncResult){
    try {
      // Get the list of friends, and sync them with the Android Contacts
      XFireFriend[] friends = XFireQuery.getFriends();
      syncContacts(context,account.name,friends);
    }
    catch (Throwable t) {
      syncResult.stats.numIoExceptions++;
    }
  }


  /***********************************************************************************************
  Adds a single contact to the platform contacts provider
  @param context the Authenticator Activity context
  @param accountName the account the contact belongs to
  @param friend the user to add
  @param rawGroupId the unique Id for the "XFire" group
  @param rawStatusId the unique Id for the "XFire" status message
  @param batchOperations any batch operations to perform when creating the new contact 
  ***********************************************************************************************/
  public static void addContact(Context context,String accountName,XFireFriend friend,long rawGroupId,long rawStatusId,BatchOperation batchOperations){
    // Creates a new ContactOperation batch process
    int userID = -1; // the raw userID will be populated when the contact is created
    ContactOperations contactOperation = ContactOperations.createNewContact(context,userID,accountName,batchOperations);

    // Adds the content to the operation
    contactOperation.addUsername(friend.getUsername());
    contactOperation.addNickname(friend.getNickname());
    contactOperation.addPhoto(friend.getPhoto());
    contactOperation.addGender(friend.getGender());
    contactOperation.addGamingStyle(friend.getGamingStyle());
    contactOperation.addLocation(friend.getLocation());
    contactOperation.addCountry(friend.getCountry());
    contactOperation.addAge(friend.getAge());
    contactOperation.addJoinDate(friend.getJoinDate());
    contactOperation.addStatus(friend.getStatus());
    contactOperation.addWebsite(friend.getWebsite());
    contactOperation.addRealName(friend.getRealName());
    contactOperation.addOccupation(friend.getOccupation());
    contactOperation.addInterests(friend.getInterests());
    contactOperation.addBiography(friend.getBiography());
    
    contactOperation.addGroupMembership(rawGroupId);
  }


  /***********************************************************************************************
  Synchronise raw contacts
  @param context The context of Authenticator Activity
  @param accountName The username for the account
  @param users The list of users
  ***********************************************************************************************/
  public static void addGroup(Context context,String accountName,BatchOperation batchOperation){

    // Creates a new ContactOperation batch process
    int groupID = -1; // the raw groupID will be populated when the group is created
    ContactOperations contactOperation = ContactOperations.createNewGroup(context,groupID,accountName,batchOperation);

    String title = context.getString(R.string.contact_group_title);
    String notes = context.getString(R.string.contact_group_notes);
    contactOperation.addGroup(title,notes);
  }


  /***********************************************************************************************
  Deletes a contact from the platform contacts provider
  @param context the Authenticator Activity context
  @param rawContactId the unique Id for this rawContact in contacts provider
  @param batchOperations any batch operations to perform when deleting the contact 
  ***********************************************************************************************/
  public static void deleteContact(Context context,long rawContactId,BatchOperation batchOperation){
    batchOperation.add(ContactOperations.addDeleteOperation(ContentUris.withAppendedId(RawContacts.CONTENT_URI,rawContactId),true).build());
  }


  /***********************************************************************************************
  Gets the platforms' rawContactID for the XFire group
  @param resolver the ContentResolver for querying the platform contacts
  @return the rawGroupID for the group, or <b>0</b> if no matching group was found
  ***********************************************************************************************/
  public static long getGroupID(ContentResolver resolver){

    String[] columns = new String[]{Groups._ID};
    String whereClause = ContactsContract.Groups.ACCOUNT_TYPE + "='" + GlobalConstants.ACCOUNT_TYPE + "'" + " AND " + ContactsContract.Groups.ACCOUNT_NAME + "='" + GlobalConstants.ACCOUNT_NAME + "'";

    Cursor c = resolver.query(ContactsContract.Groups.CONTENT_URI,columns,whereClause,null,null);

    long rawGroupID = -1;
    try {
      if (c != null && c.moveToFirst()) {
        rawGroupID = c.getLong(0);
      }
    }
    finally {
      if (c != null) {
        c.close();
      }
    }
    return rawGroupID;
  }


  /***********************************************************************************************
  Gets the platforms' rawContactID for the contact <code>XFireFriend.getUserID()</code>
  @param resolver the ContentResolver for querying the platform contacts
  @param userID the <code>XFireFriend.getUserID()</code> to lookup
  @return the rawContactID for the contact, or <b>0</b> if no matching contact was found
  ***********************************************************************************************/
  public static long getRawContactID(ContentResolver resolver,String username){

    //String[] columns = new String[]{RawContacts._ID};
    String[] columns = new String[]{Data.RAW_CONTACT_ID};
    String whereClause = RawContacts.ACCOUNT_TYPE + "='" + GlobalConstants.ACCOUNT_TYPE + "' AND " + Data.MIMETYPE + "='" + UsernameData.CONTENT_ITEM_TYPE + "' AND " + UsernameData.DATA + "='" + username + "'";

    long rawContactID = 0;
    Cursor c = resolver.query(Data.CONTENT_URI,columns,whereClause,null,null);
    try {
      if (c != null && c.moveToFirst()) {
        rawContactID = c.getLong(0);
      }
    }
    finally {
      if (c != null) {
        c.close();
      }
    }
    return rawContactID;
  }



  /***********************************************************************************************
  Synchronise raw contacts
  @param context The context of Authenticator Activity
  @param accountName The name of the account
  @param users The list of users
  ***********************************************************************************************/
  public static synchronized void syncContacts(Context context,String accountName,XFireFriend[] friends){
    if (friends == null || friends.length <= 0) {
      return;
    }

    String username = null;
    long rawContactId = 0;
    long rawGroupId = -1;
    long rawStatusId = -1;

    ContentResolver resolver = context.getContentResolver();
    BatchOperation batchOperation = new BatchOperation(context,resolver);

    rawGroupId = getGroupID(resolver);
    if (rawGroupId == -1) {
      // Create the group
      addGroup(context,accountName,batchOperation);
      // Get the group ID
      rawGroupId = getGroupID(resolver);
    }

    int friendCount = friends.length;
    for (int i = 0;i < friendCount;i++) {
      XFireFriend friend = friends[i];

      username = friend.getUsername();
      // Check to see if the contact needs to be inserted or updated
      rawContactId = getRawContactID(resolver,username);

      if (rawContactId != 0) {
        // update contact
        updateContact(context,resolver,accountName,friend,rawContactId,rawGroupId,rawStatusId,batchOperation);
      }
      else {
        // add new contact
        addContact(context,accountName,friend,rawGroupId,rawStatusId,batchOperation);
        // create the contact now, so we can update the status
        batchOperation.execute();
        // now get the rawContactID and statusDataID, then update the status
        rawContactId = getRawContactID(resolver,username);
      }

      // Execute and clear the batch operations if it gets too large
      if (batchOperation.size() >= 50) {
        batchOperation.execute();
      }
    }

    // Execute the batch operations
    batchOperation.execute();
  }


  /***********************************************************************************************
  Updates a single contact to the platform contacts provider.
  @param context the Authenticator Activity context
  @param resolver the ContentResolver to use
  @param accountName the account the contact belongs to
  @param friend the user being updated
  @param rawContactId the unique Id for this rawContact in contacts provider
  @param rawGroupId the unique Id for the "XFire" group
  @param rawStatusId the unique Id for the "XFire" status message
  ***********************************************************************************************/
  public static void updateContact(Context context,ContentResolver resolver,String accountName,XFireFriend friend,long rawContactId,long rawGroupId,long rawStatusId,BatchOperation batchOperation){
    Uri uri;




    
    
    String oldUsername = null;
    String oldNickname = null;
    //URL oldPhoto = null; // SPECIAL CASE - See lower down!
    String oldGender = null;
    String oldGamingStyle = null;
    String oldLocation = null;
    String oldCountry = null;
    int oldAge = 0;
    String oldJoinDate = null;
    String oldStatus = null;
    String oldWebsite = null;
    String oldRealName = null;
    String oldOccupation = null;
    String oldInterests = null;
    String oldBiography = null;

    long oldGroupMembership = -1;

    ContactOperations contactOperation = ContactOperations.updateExistingContact(context,rawContactId,batchOperation);

    // BUG FIX!
    // There was a bug in 1.0 and 1.1 where a new photo would be added to the contact every day!
    // This code here is to detect and remove all the photo details for the contact, if there are
    // more than 1, so that it can update the existing photo rather than adding new ones!
    String[] photoBugColumns = new String[]{Data._ID};
    String photoBugWhereClause = Data.RAW_CONTACT_ID + "=" + rawContactId + " AND " + Data.MIMETYPE + "='" + Photo.CONTENT_ITEM_TYPE + "'";
    Cursor photoBugC = resolver.query(Data.CONTENT_URI,photoBugColumns,photoBugWhereClause,null,null);
    int count = photoBugC.getCount();
    if (count > 1) {
      // remove all the photo entries
      while (photoBugC.moveToNext()) {
        long columnId = photoBugC.getLong(0);
        uri = ContentUris.withAppendedId(Data.CONTENT_URI,columnId);
        contactOperation.deletePhoto(uri);
      }
      // Clear the photos right now, so we can get the correct Photos in the next Cursor SELECT
      batchOperation.execute();
    }
    photoBugC.close();

    // Now the real update query...
    String[] columns = new String[]{Data._ID,Data.MIMETYPE,Data.DATA1,Data.DATA2,Data.DATA3,Data.DATA4,Data.DATA5,Data.DATA6,Data.DATA7,Data.DATA8,Data.DATA9,Data.DATA10,Data.SYNC1};
    String whereClause = Data.RAW_CONTACT_ID + "=" + rawContactId;
    Cursor c = resolver.query(Data.CONTENT_URI,columns,whereClause,null,null);

    boolean photoUpdated = false;
    
    try {
      while (c.moveToNext()) {
        long columnId = c.getLong(0);
        uri = ContentUris.withAppendedId(Data.CONTENT_URI,columnId);

        String mimeType = c.getString(1);

        if (mimeType.equals(UsernameData.CONTENT_ITEM_TYPE)) {
          oldUsername = c.getString(2);
          contactOperation.updateUsername(friend.getUsername(),oldUsername,uri);
        }
        else if (mimeType.equals(Nickname.CONTENT_ITEM_TYPE)) {
          oldNickname = c.getString(2);
          contactOperation.updateNickname(friend.getNickname(),oldNickname,uri);
        }
        else if (mimeType.equals(GenderData.CONTENT_ITEM_TYPE)) {
          oldGender = c.getString(2);
          contactOperation.updateGender(friend.getGender(),oldGender,uri);
        }
        else if (mimeType.equals(GamingStyleData.CONTENT_ITEM_TYPE)) {
          oldGamingStyle = c.getString(2);
          contactOperation.updateGamingStyle(friend.getGamingStyle(),oldGamingStyle,uri);
        }
        else if (mimeType.equals(LocationData.CONTENT_ITEM_TYPE)) {
          oldLocation = c.getString(2);
          contactOperation.updateLocation(friend.getLocation(),oldLocation,uri);
        }
        else if (mimeType.equals(StructuredPostal.CONTENT_ITEM_TYPE)) {
          int type = c.getInt(3);
          String label = c.getString(4);
          oldCountry = c.getString(2);
          if (type == StructuredPostal.TYPE_CUSTOM && label.equals(context.getString(R.string.label_country))) {
            contactOperation.updateCountry(friend.getCountry(),oldCountry,uri);
          }
        }
        else if (mimeType.equals(AgeData.CONTENT_ITEM_TYPE)) {
          oldAge = c.getInt(2);
          contactOperation.updateAge(friend.getAge(),oldAge,uri);
        }
        else if (mimeType.equals(JoinDateData.CONTENT_ITEM_TYPE)) {
          oldJoinDate = c.getString(2);
          contactOperation.updateJoinDate(friend.getJoinDate(),oldJoinDate,uri);
        }
        else if (mimeType.equals(StatusData.CONTENT_ITEM_TYPE)) {
          oldStatus = c.getString(2);
          contactOperation.updateStatus(friend.getStatus(),oldStatus,uri);
        }
        else if (mimeType.equals(WebsiteData.CONTENT_ITEM_TYPE)) {
          oldWebsite = c.getString(2);
          contactOperation.updateWebsite(friend.getWebsite(),oldWebsite,uri);
        }
        else if (mimeType.equals(StructuredName.CONTENT_ITEM_TYPE)) {
          oldRealName = c.getString(2);
          contactOperation.updateRealName(friend.getRealName(),oldRealName,uri);
        }
        else if (mimeType.equals(OccupationData.CONTENT_ITEM_TYPE)) {
          oldOccupation = c.getString(2);
          contactOperation.updateOccupation(friend.getOccupation(),oldOccupation,uri);
        }
        else if (mimeType.equals(InterestsData.CONTENT_ITEM_TYPE)) {
          oldInterests = c.getString(2);
          contactOperation.updateInterests(friend.getInterests(),oldInterests,uri);
        }
        else if (mimeType.equals(BiographyData.CONTENT_ITEM_TYPE)) {
          oldBiography = c.getString(2);
          contactOperation.updateBiography(friend.getBiography(),oldBiography,uri);
        }

        else if (mimeType.equals(GroupMembership.CONTENT_ITEM_TYPE)) {
          // The group never changes, so no update operation is required here. 
          // Just set the oldGroupMembership to a value so its not added to the group down below.
          oldGroupMembership = c.getLong(2);
        }
        else if (mimeType.equals(Photo.CONTENT_ITEM_TYPE)) {
          boolean timeExpired = false;
          try {
            long oldTimestamp = Long.parseLong(c.getString(12));
            long timestamp = Calendar.getInstance().getTimeInMillis();

            // Check that the photo is at least 1 day old

            long timeDifference = timestamp - oldTimestamp;
            if (timeDifference > 82800000) { // 23 hours in milliseconds
              timeExpired = true;
            }

          }
          catch (NumberFormatException e) {
            timeExpired = false;
          }

          if (timeExpired) {
            contactOperation.updatePhoto(friend.getPhoto(),uri);
          }

          // If it gets here, this contact already has a photo, so don't want to add a new photo!
          photoUpdated = true;
        }
      }
    }
    finally {
      c.close();
    }


   
    // For all unchanged values, add them from the <i>friend</i>
    if (oldUsername == null) {
      contactOperation.addUsername(friend.getUsername());
    }
    if (oldNickname == null) {
      contactOperation.addNickname(friend.getNickname());
    }
    if (oldGender == null) {
      contactOperation.addGender(friend.getGender());
    }
    if (oldGamingStyle == null) {
      contactOperation.addGamingStyle(friend.getGamingStyle());
    }
    if (oldLocation == null) {
      contactOperation.addLocation(friend.getLocation());
    }
    if (oldCountry == null) {
      contactOperation.addCountry(friend.getCountry());
    }
    if (oldAge == 0) {
      contactOperation.addAge(friend.getAge());
    }
    if (oldJoinDate == null) {
      contactOperation.addJoinDate(friend.getJoinDate());
    }
    if (oldStatus == null) {
      contactOperation.addStatus(friend.getStatus());
    }
    if (oldWebsite == null) {
      contactOperation.addWebsite(friend.getWebsite());
    }
    if (oldRealName == null) {
      contactOperation.addRealName(friend.getRealName());
    }
    if (oldOccupation == null) {
      contactOperation.addOccupation(friend.getOccupation());
    }
    if (oldInterests == null) {
      contactOperation.addInterests(friend.getInterests());
    }
    if (oldBiography == null) {
      contactOperation.addBiography(friend.getBiography());
    }

    if (oldGroupMembership == -1) {
      contactOperation.addGroupMembership(rawGroupId);
    }

    if (!photoUpdated) {
      contactOperation.addPhoto(friend.getPhoto());
    }

  }
}
